@@ -48,15 +48,19 @@ static struct cork_dllist outbound_block_list_rules;
 void
 init_block_list()
 {
-    // Initialize cache
     cache_create(&block_list, 256, NULL);
 }
 
+void
+free_block_list()
+{
+    cache_clear(block_list, 0); // Remove all items
+}
+
 int
 remove_from_block_list(char *addr)
 {
     size_t addr_len = strlen(addr);
-
     return cache_remove(block_list, addr, addr_len);
 }
 
@@ -67,7 +71,23 @@ clear_block_list()
 }
 
 int
-check_block_list(char *addr, int err_level)
+check_block_list(char *addr)
+{
+    size_t addr_len = strlen(addr);
+
+    if (cache_key_exist(block_list, addr, addr_len)) {
+        int *count = NULL;
+        cache_lookup(block_list, addr, addr_len, &count);
+
+        if (count != NULL && *count > MAX_TRIES)
+            return 1;
+    }
+
+    return 0;
+}
+
+int
+update_block_list(char *addr, int err_level)
 {
     size_t addr_len = strlen(addr);
 
@@ -79,7 +99,7 @@ check_block_list(char *addr, int err_level)
                 return 1;
             (*count) += err_level;
         }
-    } else {
+    } else if (err_level > 0) {
         int *count = (int *)ss_malloc(sizeof(int));
         *count = 1;
         cache_insert(block_list, addr, addr_len, count);
@@ -116,7 +136,7 @@ trimwhitespace(char *str)
     char *end;
 
     // Trim leading space
-    while (isspace(*str))
+    while (isspace((unsigned char)*str))
         str++;
 
     if (*str == 0)   // All spaces?
@@ -124,7 +144,7 @@ trimwhitespace(char *str)
 
     // Trim trailing space
     end = str + strlen(str) - 1;
-    while (end > str && isspace(*end))
+    while (end > str && isspace((unsigned char)*end))
         end--;
 
     // Write new null terminator
@@ -169,13 +189,12 @@ init_acl(const char *path)
                 buf[len - 1] = '\0';
             }
 
-            char *line = trimwhitespace(buf);
-
-            // Skip comments
-            if (line[0] == '#') {
-                continue;
+            char *comment = strchr(buf, '#');
+            if (comment) {
+                *comment = '\0';
             }
 
+            char *line = trimwhitespace(buf);
             if (strlen(line) == 0) {
                 continue;
             }
