@@ -69,9 +69,7 @@
 
 #include <sodium.h>
 
-#ifndef __MINGW32__
 #include <arpa/inet.h>
-#endif
 
 #include "cache.h"
 #include "encrypt.h"
@@ -251,7 +249,7 @@ int
 balloc(buffer_t *ptr, size_t capacity)
 {
     sodium_memzero(ptr, sizeof(buffer_t));
-    ptr->array    = ss_malloc(capacity);
+    ptr->data    = ss_malloc(capacity);
     ptr->capacity = capacity;
     return capacity;
 }
@@ -263,7 +261,7 @@ brealloc(buffer_t *ptr, size_t len, size_t capacity)
         return -1;
     size_t real_capacity = max(len, capacity);
     if (ptr->capacity < real_capacity) {
-        ptr->array    = ss_realloc(ptr->array, real_capacity);
+        ptr->data    = ss_realloc(ptr->data, real_capacity);
         ptr->capacity = real_capacity;
     }
     return real_capacity;
@@ -277,8 +275,8 @@ bfree(buffer_t *ptr)
     ptr->idx      = 0;
     ptr->len      = 0;
     ptr->capacity = 0;
-    if (ptr->array != NULL) {
-        ss_free(ptr->array);
+    if (ptr->data != NULL) {
+        ss_free(ptr->data);
     }
 }
 
@@ -357,7 +355,7 @@ merge(uint8_t *left, int llength, uint8_t *right,
 }
 
 static void
-merge_sort(uint8_t array[], int length,
+merge_sort(uint8_t data[], int length,
            uint32_t salt, uint64_t key)
 {
     uint8_t middle;
@@ -372,8 +370,8 @@ merge_sort(uint8_t array[], int length,
 
     llength = length - middle;
 
-    left  = array;
-    right = array + llength;
+    left  = data;
+    right = data + llength;
 
     merge_sort(left, llength, salt, key);
     merge_sort(right, middle, salt, key);
@@ -433,33 +431,39 @@ enc_table_init(const char *pass)
 }
 
 int
-cipher_iv_size(const cipher_kt_t *cipher)
+cipher_iv_size(const cipher_t *cipher)
 {
 #if defined(USE_CRYPTO_OPENSSL)
-    return EVP_CIPHER_iv_length(cipher);
+    if (cipher->info == NULL)
+        return cipher->iv_len;
+    else
+        return EVP_CIPHER_iv_length(cipher->info);
 #elif defined(USE_CRYPTO_POLARSSL) || defined(USE_CRYPTO_MBEDTLS)
     if (cipher == NULL) {
         return 0;
     }
-    return cipher->iv_size;
+    return cipher->info->iv_size;
 #endif
 }
 
 int
-cipher_key_size(const cipher_kt_t *cipher)
+cipher_key_size(const cipher_t *cipher)
 {
 #if defined(USE_CRYPTO_OPENSSL)
-    return EVP_CIPHER_key_length(cipher);
+    if (cipher->info == NULL)
+        return cipher->key_len;
+    else
+        return EVP_CIPHER_key_length(cipher->info);
 #elif defined(USE_CRYPTO_POLARSSL)
     if (cipher == NULL) {
         return 0;
     }
     /* Override PolarSSL 32 bit default key size with sane 128 bit default */
-    if (cipher->base != NULL && POLARSSL_CIPHER_ID_BLOWFISH ==
-        cipher->base->cipher) {
+    if (cipher->info->base != NULL && POLARSSL_CIPHER_ID_BLOWFISH ==
+        cipher->info->base->cipher) {
         return 128 / 8;
     }
-    return cipher->key_length / 8;
+    return cipher->info->key_length / 8;
 #elif defined(USE_CRYPTO_MBEDTLS)
     /*
      * Semi-API changes (technically public, morally private)
@@ -473,229 +477,121 @@ cipher_key_size(const cipher_kt_t *cipher)
         return 0;
     }
     /* From Version 1.2.7 released 2013-04-13 Default Blowfish keysize is now 128-bits */
-    return cipher->key_bitlen / 8;
+    return cipher->info->key_bitlen / 8;
 #endif
 }
 
 int
-bytes_to_key(const cipher_kt_t *cipher, const digest_type_t *md,
-             const uint8_t *pass, uint8_t *key, uint8_t *iv)
+bytes_to_key(const cipher_t *cipher, const digest_type_t *md,
+             const uint8_t *pass, uint8_t *key)
 {
     size_t datal;
     datal = strlen((const char *)pass);
+
 #if defined(USE_CRYPTO_OPENSSL)
-    return EVP_BytesToKey(cipher, md, NULL, pass, datal, 1, key, iv);
+
+    MD5_CTX c;
+    unsigned char md_buf[MAX_MD_SIZE];
+    int nkey;
+    int addmd;
+    unsigned int i, j, mds;
+
+    mds  = 16;
+    nkey = cipher_key_size(cipher);
+    if (pass == NULL)
+        return nkey;
+    memset(&c, 0, sizeof(MD5_CTX));
+
+    for (j = 0, addmd = 0; j < nkey; addmd++) {
+        MD5_Init(&c);
+        if (addmd) {
+            MD5_Update(&c, md_buf, mds);
+        }
+        MD5_Update(&c, pass, datal);
+        MD5_Final(md_buf, &c);
+
+        for (i = 0; i < mds; i++, j++) {
+            if (j >= nkey)
+                break;
+            key[j] = md_buf[i];
+        }
+    }
+
+    return nkey;
+
 #elif defined(USE_CRYPTO_POLARSSL)
     md_context_t c;
     unsigned char md_buf[MAX_MD_SIZE];
-    int niv;
     int nkey;
     int addmd;
-    unsigned int mds;
-    unsigned int i;
-    int rv;
+    unsigned int i, j, mds;
 
     nkey = cipher_key_size(cipher);
-    niv  = cipher_iv_size(cipher);
-    rv   = nkey;
-    if (pass == NULL) {
-        return nkey;
-    }
-
+    mds  = md_get_size(md);
     memset(&c, 0, sizeof(md_context_t));
-    if (md_init_ctx(&c, md)) {
-        return 0;
-    }
-    addmd = 0;
-    mds   = md_get_size(md);
-    for (;;) {
-        int error;
-        do {
-            error = 1;
-            if (md_starts(&c)) {
-                break;
-            }
-            if (addmd) {
-                if (md_update(&c, &(md_buf[0]), mds)) {
-                    break;
-                }
-            } else {
-                addmd = 1;
-            }
-            if (md_update(&c, pass, datal)) {
-                break;
-            }
-            if (md_finish(&c, &(md_buf[0]))) {
-                break;
-            }
-            error = 0;
-        } while (0);
-        if (error) {
-            md_free_ctx(&c);
-            memset(md_buf, 0, MAX_MD_SIZE);
-            return 0;
-        }
 
-        i = 0;
-        if (nkey) {
-            for (;;) {
-                if (nkey == 0) {
-                    break;
-                }
-                if (i == mds) {
-                    break;
-                }
-                if (key != NULL) {
-                    *(key++) = md_buf[i];
-                }
-                nkey--;
-                i++;
-            }
+    if (pass == NULL)
+        return nkey;
+    if (md_init_ctx(&c, md))
+        return 0;
+
+    for (j = 0, addmd = 0; j < nkey; addmd++) {
+        md_starts(&c);
+        if (addmd) {
+            md_update(&c, md_buf, mds);
         }
-        if (niv && (i != mds)) {
-            for (;;) {
-                if (niv == 0) {
-                    break;
-                }
-                if (i == mds) {
-                    break;
-                }
-                if (iv != NULL) {
-                    *(iv++) = md_buf[i];
-                }
-                niv--;
-                i++;
-            }
-        }
-        if ((nkey == 0) && (niv == 0)) {
-            break;
+        md_update(&c, pass, datal);
+        md_finish(&c, md_buf);
+
+        for (i = 0; i < mds; i++, j++) {
+            if (j >= nkey)
+                break;
+            key[j] = md_buf[i];
         }
     }
+
     md_free_ctx(&c);
-    memset(md_buf, 0, MAX_MD_SIZE);
-    return rv;
+    return nkey;
+
 #elif defined(USE_CRYPTO_MBEDTLS)
-/*
- *
- * Generic message digest context.
- *
- * typedef struct {
- *  Information about the associated message digest
- *  const mbedtls_md_info_t *md_info;
- *
- *  Digest-specific context
- *  void *md_ctx;
- *
- *   HMAC part of the context
- *  void *hmac_ctx;
- * } mbedtls_md_context_t; // mbedtls 2.0.0
- *
- * typedef struct {
- *  Information about the associated message digest
- *  const md_info_t *md_info;
- *
- *  Digest-specific context
- *  void *md_ctx;
- * } md_context_t; //polarssl 1.3
- *
- */
-    // NOTE: different struct body, initialize new param hmac 0 to disable HMAC
+
     mbedtls_md_context_t c;
     unsigned char md_buf[MAX_MD_SIZE];
-    int niv;
     int nkey;
     int addmd;
-    unsigned int mds;
-    unsigned int i;
-    int rv;
+    unsigned int i, j, mds;
 
     nkey = cipher_key_size(cipher);
-    niv  = cipher_iv_size(cipher);
-    rv   = nkey;
-    if (pass == NULL) {
-        return nkey;
-    }
-
+    mds  = mbedtls_md_get_size(md);
     memset(&c, 0, sizeof(mbedtls_md_context_t));
-    // XXX: md_init_ctx superseded by mbedtls_md_setup() in 2.0.0
-    // new param hmac      0 to save some memory if HMAC will not be used,
-    //                     non-zero is HMAC is going to be used with this context.
-    if (mbedtls_md_setup(&c, md, 1)) {
-        return 0;
-    }
-    addmd = 0;
-    mds   = mbedtls_md_get_size(md);
-    for (;;) {
-        int error;
-        do {
-            error = 1;
-            if (mbedtls_md_starts(&c)) {
-                break;
-            }
-            if (addmd) {
-                if (mbedtls_md_update(&c, &(md_buf[0]), mds)) {
-                    break;
-                }
-            } else {
-                addmd = 1;
-            }
-            if (mbedtls_md_update(&c, pass, datal)) {
-                break;
-            }
-            if (mbedtls_md_finish(&c, &(md_buf[0]))) {
-                break;
-            }
-            error = 0;
-        } while (0);
-        if (error) {
-            mbedtls_md_free(&c); // md_free_ctx deprecated, Use mbedtls_md_free() instead
-            memset(md_buf, 0, MAX_MD_SIZE);
-            return 0;
-        }
 
-        i = 0;
-        if (nkey) {
-            for (;;) {
-                if (nkey == 0) {
-                    break;
-                }
-                if (i == mds) {
-                    break;
-                }
-                if (key != NULL) {
-                    *(key++) = md_buf[i];
-                }
-                nkey--;
-                i++;
-            }
+    if (pass == NULL)
+        return nkey;
+    if (mbedtls_md_setup(&c, md, 1))
+        return 0;
+
+    for (j = 0, addmd = 0; j < nkey; addmd++) {
+        mbedtls_md_starts(&c);
+        if (addmd) {
+            mbedtls_md_update(&c, md_buf, mds);
         }
-        if (niv && (i != mds)) {
-            for (;;) {
-                if (niv == 0) {
-                    break;
-                }
-                if (i == mds) {
-                    break;
-                }
-                if (iv != NULL) {
-                    *(iv++) = md_buf[i];
-                }
-                niv--;
-                i++;
-            }
-        }
-        if ((nkey == 0) && (niv == 0)) {
-            break;
+        mbedtls_md_update(&c, pass, datal);
+        mbedtls_md_finish(&c, &(md_buf[0]));
+
+        for (i = 0; i < mds; i++, j++) {
+            if (j >= nkey)
+                break;
+            key[j] = md_buf[i];
         }
     }
-    mbedtls_md_free(&c); // NOTE: md_free_ctx deprecated, Use mbedtls_md_free() instead
-    memset(md_buf, 0, MAX_MD_SIZE);
-    return rv;
+
+    mbedtls_md_free(&c);
+    return nkey;
 #endif
 }
 
 int
-rand_bytes(uint8_t *output, int len)
+rand_bytes(void *output, int len)
 {
     randombytes_buf(output, len);
     // always return success
@@ -794,14 +690,16 @@ cipher_context_init(cipher_ctx_t *ctx, int method, int enc)
     }
 #endif
 
-    cipher_evp_t *evp         = &ctx->evp;
     const cipher_kt_t *cipher = get_cipher_type(method);
+
 #if defined(USE_CRYPTO_OPENSSL)
+    ctx->evp = EVP_CIPHER_CTX_new();
+    cipher_evp_t *evp = ctx->evp;
+
     if (cipher == NULL) {
         LOGE("Cipher %s not found in OpenSSL library", ciphername);
         FATAL("Cannot initialize cipher");
     }
-    EVP_CIPHER_CTX_init(evp);
     if (!EVP_CipherInit_ex(evp, cipher, NULL, NULL, NULL, enc)) {
         LOGE("Cannot initialize cipher %s", ciphername);
         exit(EXIT_FAILURE);
@@ -815,6 +713,9 @@ cipher_context_init(cipher_ctx_t *ctx, int method, int enc)
         EVP_CIPHER_CTX_set_padding(evp, 1);
     }
 #elif defined(USE_CRYPTO_POLARSSL)
+    ctx->evp = (cipher_evp_t *)ss_malloc(sizeof(cipher_evp_t));
+    cipher_evp_t *evp = ctx->evp;
+
     if (cipher == NULL) {
         LOGE("Cipher %s not found in PolarSSL library", ciphername);
         FATAL("Cannot initialize PolarSSL cipher");
@@ -823,10 +724,10 @@ cipher_context_init(cipher_ctx_t *ctx, int method, int enc)
         FATAL("Cannot initialize PolarSSL cipher context");
     }
 #elif defined(USE_CRYPTO_MBEDTLS)
-    // XXX: mbedtls_cipher_setup future change
-    // NOTE:  Currently also clears structure. In future versions you will be required to call
-    //        mbedtls_cipher_init() on the structure first.
-    //        void mbedtls_cipher_init( mbedtls_cipher_context_t *ctx );
+    ctx->evp = ss_malloc(sizeof(cipher_evp_t));
+    memset(ctx->evp, 0, sizeof(cipher_evp_t));
+    cipher_evp_t *evp = ctx->evp;
+
     if (cipher == NULL) {
         LOGE("Cipher %s not found in mbed TLS library", ciphername);
         FATAL("Cannot initialize mbed TLS cipher");
@@ -900,7 +801,7 @@ cipher_context_set_iv(cipher_ctx_t *ctx, uint8_t *iv, size_t iv_len,
     }
 #endif
 
-    cipher_evp_t *evp = &ctx->evp;
+    cipher_evp_t *evp = ctx->evp;
     if (evp == NULL) {
         LOGE("cipher_context_set_iv(): Cipher context is null");
         return;
@@ -970,15 +871,16 @@ cipher_context_release(cipher_ctx_t *ctx)
     }
 #endif
 
-    cipher_evp_t *evp = &ctx->evp;
 #if defined(USE_CRYPTO_OPENSSL)
-    EVP_CIPHER_CTX_cleanup(evp);
+    EVP_CIPHER_CTX_free(ctx->evp);
 #elif defined(USE_CRYPTO_POLARSSL)
 // NOTE: cipher_free_ctx deprecated in PolarSSL 1.3.11
-    cipher_free_ctx(evp);
+    cipher_free_ctx(ctx->evp);
+    ss_free(ctx->evp);
 #elif defined(USE_CRYPTO_MBEDTLS)
 // NOTE: cipher_free_ctx deprecated
-    mbedtls_cipher_free(evp);
+    mbedtls_cipher_free(ctx->evp);
+    ss_free(ctx->evp);
 #endif
 }
 
@@ -995,7 +897,7 @@ cipher_context_update(cipher_ctx_t *ctx, uint8_t *output, size_t *olen,
         return (ret == kCCSuccess) ? 1 : 0;
     }
 #endif
-    cipher_evp_t *evp = &ctx->evp;
+    cipher_evp_t *evp = ctx->evp;
 #if defined(USE_CRYPTO_OPENSSL)
     int err = 0, tlen = *olen;
     err = EVP_CipherUpdate(evp, (uint8_t *)output, &tlen,
@@ -1022,16 +924,16 @@ ss_onetimeauth(buffer_t *buf, uint8_t *iv, size_t capacity)
     brealloc(buf, ONETIMEAUTH_BYTES + buf->len, capacity);
 
 #if defined(USE_CRYPTO_OPENSSL)
-    HMAC(EVP_sha1(), auth_key, enc_iv_len + enc_key_len, (uint8_t *)buf->array, buf->len, (uint8_t *)hash, NULL);
+    HMAC(EVP_sha1(), auth_key, enc_iv_len + enc_key_len, (uint8_t *)buf->data, buf->len, (uint8_t *)hash, NULL);
 #elif defined(USE_CRYPTO_MBEDTLS)
     mbedtls_md_hmac(mbedtls_md_info_from_type(
-                        MBEDTLS_MD_SHA1), auth_key, enc_iv_len + enc_key_len, (uint8_t *)buf->array, buf->len,
+                        MBEDTLS_MD_SHA1), auth_key, enc_iv_len + enc_key_len, (uint8_t *)buf->data, buf->len,
                     (uint8_t *)hash);
 #else
-    sha1_hmac(auth_key, enc_iv_len + enc_key_len, (uint8_t *)buf->array, buf->len, (uint8_t *)hash);
+    sha1_hmac(auth_key, enc_iv_len + enc_key_len, (uint8_t *)buf->data, buf->len, (uint8_t *)hash);
 #endif
 
-    memcpy(buf->array + buf->len, hash, ONETIMEAUTH_BYTES);
+    memcpy(buf->data + buf->len, hash, ONETIMEAUTH_BYTES);
     buf->len += ONETIMEAUTH_BYTES;
 
     return 0;
@@ -1047,15 +949,15 @@ ss_onetimeauth_verify(buffer_t *buf, uint8_t *iv)
     size_t len = buf->len - ONETIMEAUTH_BYTES;
 
 #if defined(USE_CRYPTO_OPENSSL)
-    HMAC(EVP_sha1(), auth_key, enc_iv_len + enc_key_len, (uint8_t *)buf->array, len, hash, NULL);
+    HMAC(EVP_sha1(), auth_key, enc_iv_len + enc_key_len, (uint8_t *)buf->data, len, hash, NULL);
 #elif defined(USE_CRYPTO_MBEDTLS)
     mbedtls_md_hmac(mbedtls_md_info_from_type(
-                        MBEDTLS_MD_SHA1), auth_key, enc_iv_len + enc_key_len, (uint8_t *)buf->array, len, hash);
+                        MBEDTLS_MD_SHA1), auth_key, enc_iv_len + enc_key_len, (uint8_t *)buf->data, len, hash);
 #else
-    sha1_hmac(auth_key, enc_iv_len + enc_key_len, (uint8_t *)buf->array, len, hash);
+    sha1_hmac(auth_key, enc_iv_len + enc_key_len, (uint8_t *)buf->data, len, hash);
 #endif
 
-    return safe_memcmp(buf->array + len, hash, ONETIMEAUTH_BYTES);
+    return safe_memcmp(buf->data + len, hash, ONETIMEAUTH_BYTES);
 }
 
 int
@@ -1077,7 +979,7 @@ ss_encrypt_all(buffer_t *plain, int method, int auth, size_t capacity)
 
         rand_bytes(iv, iv_len);
         cipher_context_set_iv(&evp, iv, iv_len, 1);
-        memcpy(cipher->array, iv, iv_len);
+        memcpy(cipher->data, iv, iv_len);
 
         if (auth) {
             ss_onetimeauth(plain, iv, capacity);
@@ -1085,13 +987,13 @@ ss_encrypt_all(buffer_t *plain, int method, int auth, size_t capacity)
         }
 
         if (method >= SALSA20) {
-            crypto_stream_xor_ic((uint8_t *)(cipher->array + iv_len),
-                                 (const uint8_t *)plain->array, (uint64_t)(plain->len),
+            crypto_stream_xor_ic((uint8_t *)(cipher->data + iv_len),
+                                 (const uint8_t *)plain->data, (uint64_t)(plain->len),
                                  (const uint8_t *)iv,
                                  0, enc_key, method);
         } else {
-            err = cipher_context_update(&evp, (uint8_t *)(cipher->array + iv_len),
-                                        &cipher->len, (const uint8_t *)plain->array,
+            err = cipher_context_update(&evp, (uint8_t *)(cipher->data + iv_len),
+                                        &cipher->len, (const uint8_t *)plain->data,
                                         plain->len);
         }
 
@@ -1102,20 +1004,20 @@ ss_encrypt_all(buffer_t *plain, int method, int auth, size_t capacity)
         }
 
 #ifdef DEBUG
-        dump("PLAIN", plain->array, plain->len);
-        dump("CIPHER", cipher->array + iv_len, cipher->len);
+        dump("PLAIN", plain->data, plain->len);
+        dump("CIPHER", cipher->data + iv_len, cipher->len);
 #endif
 
         cipher_context_release(&evp);
 
         brealloc(plain, iv_len + cipher->len, capacity);
-        memcpy(plain->array, cipher->array, iv_len + cipher->len);
+        memcpy(plain->data, cipher->data, iv_len + cipher->len);
         plain->len = iv_len + cipher->len;
 
         return 0;
     } else {
-        char *begin = plain->array;
-        char *ptr   = plain->array;
+        char *begin = plain->data;
+        char *ptr   = plain->data;
         while (ptr < begin + plain->len) {
             *ptr = (char)enc_table[(uint8_t)*ptr];
             ptr++;
@@ -1142,7 +1044,7 @@ ss_encrypt(buffer_t *plain, enc_ctx_t *ctx, size_t capacity)
 
         if (!ctx->init) {
             cipher_context_set_iv(&ctx->evp, ctx->evp.iv, iv_len, 1);
-            memcpy(cipher->array, ctx->evp.iv, iv_len);
+            memcpy(cipher->data, ctx->evp.iv, iv_len);
             ctx->counter = 0;
             ctx->init    = 1;
         }
@@ -1152,25 +1054,25 @@ ss_encrypt(buffer_t *plain, enc_ctx_t *ctx, size_t capacity)
             brealloc(cipher, iv_len + (padding + cipher->len) * 2, capacity);
             if (padding) {
                 brealloc(plain, plain->len + padding, capacity);
-                memmove(plain->array + padding, plain->array, plain->len);
-                sodium_memzero(plain->array, padding);
+                memmove(plain->data + padding, plain->data, plain->len);
+                sodium_memzero(plain->data, padding);
             }
-            crypto_stream_xor_ic((uint8_t *)(cipher->array + iv_len),
-                                 (const uint8_t *)plain->array,
+            crypto_stream_xor_ic((uint8_t *)(cipher->data + iv_len),
+                                 (const uint8_t *)plain->data,
                                  (uint64_t)(plain->len + padding),
                                  (const uint8_t *)ctx->evp.iv,
                                  ctx->counter / SODIUM_BLOCK_SIZE, enc_key,
                                  enc_method);
             ctx->counter += plain->len;
             if (padding) {
-                memmove(cipher->array + iv_len,
-                        cipher->array + iv_len + padding, cipher->len);
+                memmove(cipher->data + iv_len,
+                        cipher->data + iv_len + padding, cipher->len);
             }
         } else {
             err =
                 cipher_context_update(&ctx->evp,
-                                      (uint8_t *)(cipher->array + iv_len),
-                                      &cipher->len, (const uint8_t *)plain->array,
+                                      (uint8_t *)(cipher->data + iv_len),
+                                      &cipher->len, (const uint8_t *)plain->data,
                                       plain->len);
             if (!err) {
                 return -1;
@@ -1178,18 +1080,18 @@ ss_encrypt(buffer_t *plain, enc_ctx_t *ctx, size_t capacity)
         }
 
 #ifdef DEBUG
-        dump("PLAIN", plain->array, plain->len);
-        dump("CIPHER", cipher->array + iv_len, cipher->len);
+        dump("PLAIN", plain->data, plain->len);
+        dump("CIPHER", cipher->data + iv_len, cipher->len);
 #endif
 
         brealloc(plain, iv_len + cipher->len, capacity);
-        memcpy(plain->array, cipher->array, iv_len + cipher->len);
+        memcpy(plain->data, cipher->data, iv_len + cipher->len);
         plain->len = iv_len + cipher->len;
 
         return 0;
     } else {
-        char *begin = plain->array;
-        char *ptr   = plain->array;
+        char *begin = plain->data;
+        char *ptr   = plain->data;
         while (ptr < begin + plain->len) {
             *ptr = (char)enc_table[(uint8_t)*ptr];
             ptr++;
@@ -1218,21 +1120,21 @@ ss_decrypt_all(buffer_t *cipher, int method, int auth, size_t capacity)
         plain->len = cipher->len - iv_len;
 
         uint8_t iv[MAX_IV_LENGTH];
-        memcpy(iv, cipher->array, iv_len);
+        memcpy(iv, cipher->data, iv_len);
         cipher_context_set_iv(&evp, iv, iv_len, 0);
 
         if (method >= SALSA20) {
-            crypto_stream_xor_ic((uint8_t *)plain->array,
-                                 (const uint8_t *)(cipher->array + iv_len),
+            crypto_stream_xor_ic((uint8_t *)plain->data,
+                                 (const uint8_t *)(cipher->data + iv_len),
                                  (uint64_t)(cipher->len - iv_len),
                                  (const uint8_t *)iv, 0, enc_key, method);
         } else {
-            ret = cipher_context_update(&evp, (uint8_t *)plain->array, &plain->len,
-                                        (const uint8_t *)(cipher->array + iv_len),
+            ret = cipher_context_update(&evp, (uint8_t *)plain->data, &plain->len,
+                                        (const uint8_t *)(cipher->data + iv_len),
                                         cipher->len - iv_len);
         }
 
-        if (auth || (plain->array[0] & ONETIMEAUTH_FLAG)) {
+        if (auth || (plain->data[0] & ONETIMEAUTH_FLAG)) {
             if (plain->len > ONETIMEAUTH_BYTES) {
                 ret = !ss_onetimeauth_verify(plain, iv);
                 if (ret) {
@@ -1250,20 +1152,20 @@ ss_decrypt_all(buffer_t *cipher, int method, int auth, size_t capacity)
         }
 
 #ifdef DEBUG
-        dump("PLAIN", plain->array, plain->len);
-        dump("CIPHER", cipher->array + iv_len, cipher->len - iv_len);
+        dump("PLAIN", plain->data, plain->len);
+        dump("CIPHER", cipher->data + iv_len, cipher->len - iv_len);
 #endif
 
         cipher_context_release(&evp);
 
         brealloc(cipher, plain->len, capacity);
-        memcpy(cipher->array, plain->array, plain->len);
+        memcpy(cipher->data, plain->data, plain->len);
         cipher->len = plain->len;
 
         return 0;
     } else {
-        char *begin = cipher->array;
-        char *ptr   = cipher->array;
+        char *begin = cipher->data;
+        char *ptr   = cipher->data;
         while (ptr < begin + cipher->len) {
             *ptr = (char)dec_table[(uint8_t)*ptr];
             ptr++;
@@ -1290,7 +1192,7 @@ ss_decrypt(buffer_t *cipher, enc_ctx_t *ctx, size_t capacity)
             iv_len      = enc_iv_len;
             plain->len -= iv_len;
 
-            memcpy(iv, cipher->array, iv_len);
+            memcpy(iv, cipher->data, iv_len);
             cipher_context_set_iv(&ctx->evp, iv, iv_len, 0);
             ctx->counter = 0;
             ctx->init    = 1;
@@ -1311,23 +1213,23 @@ ss_decrypt(buffer_t *cipher, enc_ctx_t *ctx, size_t capacity)
 
             if (padding) {
                 brealloc(cipher, cipher->len + padding, capacity);
-                memmove(cipher->array + iv_len + padding, cipher->array + iv_len,
+                memmove(cipher->data + iv_len + padding, cipher->data + iv_len,
                         cipher->len - iv_len);
-                sodium_memzero(cipher->array + iv_len, padding);
+                sodium_memzero(cipher->data + iv_len, padding);
             }
-            crypto_stream_xor_ic((uint8_t *)plain->array,
-                                 (const uint8_t *)(cipher->array + iv_len),
+            crypto_stream_xor_ic((uint8_t *)plain->data,
+                                 (const uint8_t *)(cipher->data + iv_len),
                                  (uint64_t)(cipher->len - iv_len + padding),
                                  (const uint8_t *)ctx->evp.iv,
                                  ctx->counter / SODIUM_BLOCK_SIZE, enc_key,
                                  enc_method);
             ctx->counter += cipher->len - iv_len;
             if (padding) {
-                memmove(plain->array, plain->array + padding, plain->len);
+                memmove(plain->data, plain->data + padding, plain->len);
             }
         } else {
-            err = cipher_context_update(&ctx->evp, (uint8_t *)plain->array, &plain->len,
-                                        (const uint8_t *)(cipher->array + iv_len),
+            err = cipher_context_update(&ctx->evp, (uint8_t *)plain->data, &plain->len,
+                                        (const uint8_t *)(cipher->data + iv_len),
                                         cipher->len - iv_len);
         }
 
@@ -1337,18 +1239,18 @@ ss_decrypt(buffer_t *cipher, enc_ctx_t *ctx, size_t capacity)
         }
 
 #ifdef DEBUG
-        dump("PLAIN", plain->array, plain->len);
-        dump("CIPHER", cipher->array + iv_len, cipher->len - iv_len);
+        dump("PLAIN", plain->data, plain->len);
+        dump("CIPHER", cipher->data + iv_len, cipher->len - iv_len);
 #endif
 
         brealloc(cipher, plain->len, capacity);
-        memcpy(cipher->array, plain->array, plain->len);
+        memcpy(cipher->data, plain->data, plain->len);
         cipher->len = plain->len;
 
         return 0;
     } else {
-        char *begin = cipher->array;
-        char *ptr   = cipher->array;
+        char *begin = cipher->data;
+        char *ptr   = cipher->data;
         while (ptr < begin + cipher->len) {
             *ptr = (char)dec_table[(uint8_t)*ptr];
             ptr++;
@@ -1381,12 +1283,12 @@ enc_key_init(int method, const char *pass)
 
 #if defined(USE_CRYPTO_OPENSSL)
     OpenSSL_add_all_algorithms();
+#else
+    cipher_kt_t cipher_info;
 #endif
 
-    uint8_t iv[MAX_IV_LENGTH];
-
-    cipher_kt_t *cipher;
-    cipher_kt_t cipher_info;
+    cipher_t cipher;
+    memset(&cipher, 0, sizeof(cipher_t));
 
     // Initialize sodium for random generator
     if (sodium_init() == -1) {
@@ -1394,35 +1296,36 @@ enc_key_init(int method, const char *pass)
     }
 
     if (method == SALSA20 || method == CHACHA20 || method == CHACHA20IETF) {
-        // Fake cipher
-        cipher = (cipher_kt_t *)&cipher_info;
 #if defined(USE_CRYPTO_OPENSSL)
-        cipher->key_len = supported_ciphers_key_size[method];
-        cipher->iv_len  = supported_ciphers_iv_size[method];
+        cipher.info    = NULL;
+        cipher.key_len = supported_ciphers_key_size[method];
+        cipher.iv_len  = supported_ciphers_iv_size[method];
 #endif
 #if defined(USE_CRYPTO_POLARSSL)
-        cipher->base       = NULL;
-        cipher->key_length = supported_ciphers_key_size[method] * 8;
-        cipher->iv_size    = supported_ciphers_iv_size[method];
+        cipher.info             = &cipher_info;
+        cipher.info->base       = NULL;
+        cipher.info->key_length = supported_ciphers_key_size[method] * 8;
+        cipher.info->iv_size    = supported_ciphers_iv_size[method];
 #endif
 #if defined(USE_CRYPTO_MBEDTLS)
         // XXX: key_length changed to key_bitlen in mbed TLS 2.0.0
-        cipher->base       = NULL;
-        cipher->key_bitlen = supported_ciphers_key_size[method] * 8;
-        cipher->iv_size    = supported_ciphers_iv_size[method];
+        cipher.info             = &cipher_info;
+        cipher.info->base       = NULL;
+        cipher.info->key_bitlen = supported_ciphers_key_size[method] * 8;
+        cipher.info->iv_size    = supported_ciphers_iv_size[method];
 #endif
     } else {
-        cipher = (cipher_kt_t *)get_cipher_type(method);
+        cipher.info = (cipher_kt_t *)get_cipher_type(method);
     }
 
-    if (cipher == NULL) {
+    if (cipher.info == NULL && cipher.key_len == 0) {
         do {
 #if defined(USE_CRYPTO_POLARSSL) && defined(USE_CRYPTO_APPLECC)
             if (supported_ciphers_applecc[method] != kCCAlgorithmInvalid) {
                 cipher_info.base       = NULL;
                 cipher_info.key_length = supported_ciphers_key_size[method] * 8;
                 cipher_info.iv_size    = supported_ciphers_iv_size[method];
-                cipher                 = (cipher_kt_t *)&cipher_info;
+                cipher.info            = (cipher_kt_t *)&cipher_info;
                 break;
             }
 #endif
@@ -1432,12 +1335,11 @@ enc_key_init(int method, const char *pass)
                 cipher_info.base       = NULL;
                 cipher_info.key_bitlen = supported_ciphers_key_size[method] * 8;
                 cipher_info.iv_size    = supported_ciphers_iv_size[method];
-                cipher                 = (cipher_kt_t *)&cipher_info;
+                cipher.info            = (cipher_kt_t *)&cipher_info;
                 break;
             }
 #endif
-            LOGE("Cipher %s not found in crypto library",
-                 supported_ciphers[method]);
+            LOGE("Cipher %s not found in crypto library", supported_ciphers[method]);
             FATAL("Cannot initialize cipher");
         } while (0);
     }
@@ -1447,14 +1349,15 @@ enc_key_init(int method, const char *pass)
         FATAL("MD5 Digest not found in crypto library");
     }
 
-    enc_key_len = bytes_to_key(cipher, md, (const uint8_t *)pass, enc_key, iv);
+    enc_key_len = bytes_to_key(&cipher, md, (const uint8_t *)pass, enc_key);
+
     if (enc_key_len == 0) {
         FATAL("Cannot generate key and IV");
     }
     if (method == RC4_MD5) {
         enc_iv_len = 16;
     } else {
-        enc_iv_len = cipher_iv_size(cipher);
+        enc_iv_len = cipher_iv_size(&cipher);
     }
     enc_method = method;
 }
@@ -1492,10 +1395,10 @@ ss_check_hash(buffer_t *buf, chunk_t *chunk, enc_ctx_t *ctx, size_t capacity)
     brealloc(buf, chunk->len + blen, capacity);
 
     for (i = 0, j = 0, k = 0; i < blen; i++) {
-        chunk->buf->array[cidx++] = buf->array[k++];
+        chunk->buf->data[cidx++] = buf->data[k++];
 
         if (cidx == CLEN_BYTES) {
-            uint16_t clen = ntohs(*((uint16_t *)chunk->buf->array));
+            uint16_t clen = ntohs(*((uint16_t *)chunk->buf->data));
             brealloc(chunk->buf, clen + AUTH_BYTES, capacity);
             chunk->len = clen;
         }
@@ -1510,22 +1413,22 @@ ss_check_hash(buffer_t *buf, chunk_t *chunk, enc_ctx_t *ctx, size_t capacity)
             memcpy(key + enc_iv_len, &c, sizeof(uint32_t));
 #if defined(USE_CRYPTO_OPENSSL)
             HMAC(EVP_sha1(), key, enc_iv_len + sizeof(uint32_t),
-                 (uint8_t *)chunk->buf->array + AUTH_BYTES, chunk->len, hash, NULL);
+                 (uint8_t *)chunk->buf->data + AUTH_BYTES, chunk->len, hash, NULL);
 #elif defined(USE_CRYPTO_MBEDTLS)
             mbedtls_md_hmac(mbedtls_md_info_from_type(MBEDTLS_MD_SHA1), key, enc_iv_len + sizeof(uint32_t),
-                            (uint8_t *)chunk->buf->array + AUTH_BYTES, chunk->len, hash);
+                            (uint8_t *)chunk->buf->data + AUTH_BYTES, chunk->len, hash);
 #else
             sha1_hmac(key, enc_iv_len + sizeof(uint32_t),
-                      (uint8_t *)chunk->buf->array + AUTH_BYTES, chunk->len, hash);
+                      (uint8_t *)chunk->buf->data + AUTH_BYTES, chunk->len, hash);
 #endif
 
-            if (safe_memcmp(hash, chunk->buf->array + CLEN_BYTES, ONETIMEAUTH_BYTES) != 0) {
+            if (safe_memcmp(hash, chunk->buf->data + CLEN_BYTES, ONETIMEAUTH_BYTES) != 0) {
                 return 0;
             }
 
             // Copy chunk back to buffer
-            memmove(buf->array + j + chunk->len, buf->array + k, blen - i - 1);
-            memcpy(buf->array + j, chunk->buf->array + AUTH_BYTES, chunk->len);
+            memmove(buf->data + j + chunk->len, buf->data + k, blen - i - 1);
+            memcpy(buf->data + j, chunk->buf->data + AUTH_BYTES, chunk->len);
 
             // Reset the base offset
             j   += chunk->len;
@@ -1553,17 +1456,17 @@ ss_gen_hash(buffer_t *buf, uint32_t *counter, enc_ctx_t *ctx, size_t capacity)
     memcpy(key, ctx->evp.iv, enc_iv_len);
     memcpy(key + enc_iv_len, &c, sizeof(uint32_t));
 #if defined(USE_CRYPTO_OPENSSL)
-    HMAC(EVP_sha1(), key, enc_iv_len + sizeof(uint32_t), (uint8_t *)buf->array, blen, hash, NULL);
+    HMAC(EVP_sha1(), key, enc_iv_len + sizeof(uint32_t), (uint8_t *)buf->data, blen, hash, NULL);
 #elif defined(USE_CRYPTO_MBEDTLS)
     mbedtls_md_hmac(mbedtls_md_info_from_type(
-                        MBEDTLS_MD_SHA1), key, enc_iv_len + sizeof(uint32_t), (uint8_t *)buf->array, blen, hash);
+                        MBEDTLS_MD_SHA1), key, enc_iv_len + sizeof(uint32_t), (uint8_t *)buf->data, blen, hash);
 #else
-    sha1_hmac(key, enc_iv_len + sizeof(uint32_t), (uint8_t *)buf->array, blen, hash);
+    sha1_hmac(key, enc_iv_len + sizeof(uint32_t), (uint8_t *)buf->data, blen, hash);
 #endif
 
-    memmove(buf->array + AUTH_BYTES, buf->array, blen);
-    memcpy(buf->array + CLEN_BYTES, hash, ONETIMEAUTH_BYTES);
-    memcpy(buf->array, &chunk_len, CLEN_BYTES);
+    memmove(buf->data + AUTH_BYTES, buf->data, blen);
+    memcpy(buf->data + CLEN_BYTES, hash, ONETIMEAUTH_BYTES);
+    memcpy(buf->data, &chunk_len, CLEN_BYTES);
 
     *counter = *counter + 1;
     buf->len = blen + AUTH_BYTES;
