@@ -23,20 +23,7 @@
 #ifndef _ENCRYPT_H
 #define _ENCRYPT_H
 
-#ifndef __MINGW32__
 #include <sys/socket.h>
-#else
-
-#ifdef max
-#undef max
-#endif
-
-#ifdef min
-#undef min
-#endif
-
-#endif
-
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
