@@ -34,13 +34,11 @@
 #include <unistd.h>
 #include <getopt.h>
 
-#ifndef __MINGW32__
 #include <errno.h>
 #include <arpa/inet.h>
 #include <netdb.h>
 #include <netinet/in.h>
 #include <pthread.h>
-#endif
 
 #ifdef LIB_ONLY
 #include <pthread.h>
@@ -56,10 +54,6 @@
 #include <libcork/core.h>
 #include <udns.h>
 
-#ifdef __MINGW32__
-#include "win32.h"
-#endif
-
 #include "netutils.h"
 #include "utils.h"
 #include "socks5.h"
@@ -99,7 +93,6 @@ int vpn        = 0;
 uint64_t tx    = 0;
 uint64_t rx    = 0;
 ev_tstamp last = 0;
-char *prefix;
 #endif
 
 static int acl       = 0;
@@ -110,10 +103,8 @@ static int fast_open = 0;
 
 static struct ev_signal sigint_watcher;
 static struct ev_signal sigterm_watcher;
-#ifndef __MINGW32__
 static struct ev_signal sigchld_watcher;
 static struct ev_signal sigusr1_watcher;
-#endif
 
 #ifdef HAVE_SETRLIMIT
 #ifndef LIB_ONLY
@@ -143,7 +134,6 @@ static server_t *new_server(int fd, int method);
 
 static struct cork_dllist connections;
 
-#ifndef __MINGW32__
 int
 setnonblocking(int fd)
 {
@@ -153,7 +143,6 @@ setnonblocking(int fd)
     }
     return fcntl(fd, F_SETFL, flags | O_NONBLOCK);
 }
-#endif
 
 int
 create_and_bind(const char *addr, const char *port)
@@ -1130,22 +1119,18 @@ signal_cb(EV_P_ ev_signal *w, int revents)
 {
     if (revents & EV_SIGNAL) {
         switch (w->signum) {
-#ifndef __MINGW32__
         case SIGCHLD:
             if (!is_plugin_running())
                 LOGE("plugin service exit unexpectedly");
             else
                 return;
         case SIGUSR1:
-#endif
         case SIGINT:
         case SIGTERM:
             ev_signal_stop(EV_DEFAULT, &sigint_watcher);
             ev_signal_stop(EV_DEFAULT, &sigterm_watcher);
-#ifndef __MINGW32__
             ev_signal_stop(EV_DEFAULT, &sigchld_watcher);
             ev_signal_stop(EV_DEFAULT, &sigusr1_watcher);
-#endif
             keep_resolving = 0;
             ev_unloop(EV_A_ EVUNLOOP_ALL);
         }
@@ -1221,7 +1206,7 @@ main(int argc, char **argv)
     USE_TTY();
 
 #ifdef ANDROID
-    while ((c = getopt_long(argc, argv, "f:s:p:l:k:t:m:i:c:b:a:n:P:huUvVA6",
+    while ((c = getopt_long(argc, argv, "f:s:p:l:k:t:m:i:c:b:a:n:huUvVA6",
                             long_options, &option_index)) != -1) {
 #else
     while ((c = getopt_long(argc, argv, "f:s:p:l:k:t:m:i:c:b:a:n:huUvA6",
@@ -1313,9 +1298,6 @@ main(int argc, char **argv)
         case 'V':
             vpn = 1;
             break;
-        case 'P':
-            prefix = optarg;
-            break;
 #endif
         case '?':
             // The option character is not recognized.
@@ -1477,13 +1459,9 @@ main(int argc, char **argv)
         }
     }
 
-#ifdef __MINGW32__
-    winsock_init();
-#else
     // ignore SIGPIPE
     signal(SIGPIPE, SIG_IGN);
     signal(SIGABRT, SIG_IGN);
-#endif
 
     // Setup keys
     LOGI("initializing ciphers... %s", method);
@@ -1521,11 +1499,8 @@ main(int argc, char **argv)
     ev_signal_init(&sigterm_watcher, signal_cb, SIGTERM);
     ev_signal_start(EV_DEFAULT, &sigint_watcher);
     ev_signal_start(EV_DEFAULT, &sigterm_watcher);
-
-#ifndef __MINGW32__
     ev_signal_init(&sigchld_watcher, signal_cb, SIGCHLD);
     ev_signal_start(EV_DEFAULT, &sigchld_watcher);
-#endif
 
     struct ev_loop *loop = EV_DEFAULT;
 
@@ -1581,11 +1556,9 @@ main(int argc, char **argv)
         FATAL("failed to switch user");
     }
 
-#ifndef __MINGW32__
     if (geteuid() == 0) {
         LOGI("running from root user");
     }
-#endif
 
     // Init connections
     cork_dllist_init(&connections);
@@ -1615,10 +1588,6 @@ main(int argc, char **argv)
         free_udprelay();
     }
 
-#ifdef __MINGW32__
-    winsock_cleanup();
-#endif
-
     return 0;
 }
 
@@ -1662,24 +1631,18 @@ start_ss_local_server(profile_t profile)
         local_addr = "127.0.0.1";
     }
 
-#ifdef __MINGW32__
-    winsock_init();
-#else
     // ignore SIGPIPE
     signal(SIGPIPE, SIG_IGN);
     signal(SIGABRT, SIG_IGN);
-#endif
 
     ev_signal_init(&sigint_watcher, signal_cb, SIGINT);
     ev_signal_init(&sigterm_watcher, signal_cb, SIGTERM);
     ev_signal_start(EV_DEFAULT, &sigint_watcher);
     ev_signal_start(EV_DEFAULT, &sigterm_watcher);
-#ifndef __MINGW32__
     ev_signal_init(&sigusr1_watcher, signal_cb, SIGUSR1);
     ev_signal_init(&sigchld_watcher, signal_cb, SIGCHLD);
     ev_signal_start(EV_DEFAULT, &sigusr1_watcher);
     ev_signal_start(EV_DEFAULT, &sigchld_watcher);
-#endif
 
     // Setup keys
     LOGI("initializing ciphers... %s", method);
@@ -1761,10 +1724,6 @@ start_ss_local_server(profile_t profile)
     ss_free(listen_ctx.remote_addr[0]);
     ss_free(remote_addr_tmp);
 
-#ifdef __MINGW32__
-    winsock_cleanup();
-#endif
-
     // cannot reach here
     return 0;
 }
